@@ -1,5 +1,6 @@
 //! `GNSS` constellations & associated methods
 use thiserror::Error;
+use serde_derive::{Deserialize, Serialize};
 
 const GPS_STR_IDENTIFIER     : &str = "GPS";
 const GLONASS_STR_IDENTIFIER : &str = "GLO"; 
@@ -10,23 +11,32 @@ const SBAS_STR_IDENTIFIER    : &str = "SBS";
 const MIXED_STR_IDENTIFIER   : &str = "M";
 
 /// Number of known ̀`GNSS` constellations
-pub const CONSTELLATION_NUMBER: u8 = 6;
+pub const CONSTELLATION_LENGTH: usize = 6;
 
 /// Describes all known `GNSS` constellations
 /// when manipulating `RINEX`
 #[derive(Clone, Copy, PartialEq, Eq, Hash, Debug)]
+#[derive(Serialize, Deserialize)]
 pub enum Constellation {
+    /// `GPS` american constellation,
     GPS,
+    /// `Glonass` russian constellation
     Glonass,
+    /// `Beidou` chinese constellation
     Beidou,
+    /// `QZSS` japanese constellation
     QZSS,
+    /// `Galileo` european constellation
     Galileo,
+    /// `Sbas` constellation
     Sbas,
+    /// `Mixed` for Mixed constellations 
+    /// RINEX files description
     Mixed,
 }
 
 impl Default for Constellation {
-    /// Builds a default `GNSS` constellation
+    /// Builds a default `GNSS::GPS` constellation
     fn default() -> Constellation {
         Constellation::GPS
     }
