@@ -5,7 +5,8 @@ use std::collections::HashMap;
 
 use crate::version;
 use crate::gnss_time;
-use crate::{is_comment, Type, TypeError};
+use crate::{is_comment};
+use crate::types::{Type, TypeError};
 use crate::constellation;
 use crate::constellation::{Constellation, ConstellationError};
 
@@ -17,8 +18,11 @@ pub const HEADER_END_MARKER : &str = "END OF HEADER";
 /// GNSS receiver description
 #[derive(Debug)]
 pub struct Rcvr {
+    /// Receiver (hardware) model
     model: String, 
+    /// Receiver (hardware) identification info
     sn: String, // serial #
+    /// Receiver embedded software info
     firmware: String, // firmware #
 }
 
@@ -77,17 +81,17 @@ impl Sensor {
 #[derive(Debug)]
 pub struct Antenna {
     /// Hardware model / make descriptor
-    model: String,
+    pub model: String,
     /// Serial number / identification number
-    sn: String,
+    pub sn: String,
     /// 3D coordinates of reference point
-    coords: Option<rust_3d::Point3D>,
+    pub coords: Option<rust_3d::Point3D>,
     /// height in comparison to ref. point
-    height: Option<f32>,
+    pub height: Option<f32>,
     /// eastern eccentricity compared to ref. point
-    eastern_eccentricity: Option<f32>,
+    pub eastern_eccentricity: Option<f32>,
     /// northern eccentricity compare to ref. point
-    northern_eccentricity: Option<f32>,
+    pub northern_eccentricity: Option<f32>,
 }
 
 impl Default for Antenna {
@@ -121,14 +125,18 @@ impl std::str::FromStr for Antenna {
 }
 
 impl Antenna {
-    /// Assigns antenna coordinates
-    pub fn set_coords (&mut self, coords: rust_3d::Point3D) { self.coords = Some(coords) }
-    /// Sets Antenna height compared to reference point
-    pub fn set_height (&mut self, height: f32) { self.height = Some(height) }
-    /// Sets Eastern eccentricity (m)
-    pub fn set_eastern_eccentricity (&mut self, e: f32) { self.eastern_eccentricity = Some(e) }
-    /// Sets Northern eccentricity (m)
-    pub fn set_northern_eccentricity (&mut self, e: f32) { self.northern_eccentricity = Some(e) }
+    pub fn new (sn: &str, model: &str, 
+        coords: Option<rust_3d::Point3D>,
+            h : Option<f32>, e: Option<f32>, n: Option<f32>) -> Antenna {
+        Antenna {
+            sn: sn.to_string(),
+            model: model.to_string(),
+            coords: coords,
+            height: h,
+            northern_eccentricity: n,
+            eastern_eccentricity: e,
+        }
+    }
 }
 
 /// `LeapSecond` to describe leap seconds.
@@ -186,11 +194,11 @@ impl std::str::FromStr for LeapSecond {
 }
 
 /// Describes `Compact RINEX` specific information
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct CrinexInfo {
-    version: String, // compression version
-    prog: String, // compression program
-    date: chrono::NaiveDateTime, // date of compression
+    pub version: version::Version, // compression version
+    pub prog: String, // compression program
+    pub date: chrono::NaiveDateTime, // date of compression
 }
 
 /// Describes known marker types
@@ -227,7 +235,7 @@ enum MarkerType {
 
 /// Describes `RINEX` file header
 #[derive(Debug)]
-pub struct RinexHeader {
+pub struct Header {
     /// revision for this `RINEX`
     pub version: version::Version, 
     /// optionnal `CRINEX` (compressed `RINEX` infos), 
@@ -252,6 +260,9 @@ pub struct RinexHeader {
     pub observer: String, 
     /// name of production agency
     pub agency: String, 
+    /// optionnal comments encountered in header section, 
+    /// exposed as is
+    pub comments : Vec<String>,
     /// optionnal hardware (receiver) infos
     pub rcvr: Option<Rcvr>, 
     /// optionnal antenna infos
@@ -317,9 +328,9 @@ pub enum Error {
     LeapSecondParsingError(String),
 }
 
-impl Default for RinexHeader {
-    fn default() -> RinexHeader {
-        RinexHeader {
+impl Default for Header {
+    fn default() -> Header {
+        Header {
             version: version::Version::default(), 
             crinex: None,
             rinex_type: Type::default(),
@@ -331,6 +342,7 @@ impl Default for RinexHeader {
             observer: String::from("Unknown"),
             agency: String::from("Unknown"),
             station_url: None,
+            comments: Vec::with_capacity(4),
             leap: None,
             license: None,
             doi: None,
@@ -355,15 +367,17 @@ impl Default for RinexHeader {
     }
 }
 
-impl std::str::FromStr for RinexHeader {
+impl std::str::FromStr for Header {
     type Err = Error;
     /// Builds header from extracted header description
     fn from_str (content: &str) -> Result<Self, Self::Err> {
+        let mut comments : Vec<String> = Vec::with_capacity(4);
         let mut lines = content.lines();
         let mut line = lines.next()
             .unwrap();
         // comments ?
         while is_comment!(line) {
+            comments.push(String::from(line.split_at(60).0));
             line = lines.next()
                 .unwrap()
         }
@@ -380,7 +394,7 @@ impl std::str::FromStr for RinexHeader {
                 let date = remainder.split_at(20).0.trim();
                 println!("CRINEX: VERSION \"{}\" | PGM \"{}\" | DATE \"{}\"", version.trim(), pgm.trim(), date); 
                 Some(CrinexInfo {
-                    version: version.trim().to_string(),
+                    version: version::Version::from_str(version.trim())?,
                     prog: pgm.trim().to_string(),
                     date: chrono::NaiveDateTime::parse_from_str(date, "%d-%b-%y %H:%M")?
                 })
@@ -393,6 +407,7 @@ impl std::str::FromStr for RinexHeader {
         }
         // comments ?
         while is_comment!(line) {
+            comments.push(String::from(line.split_at(60).0));
             line = lines.next()
                 .unwrap()
         }
@@ -413,7 +428,6 @@ impl std::str::FromStr for RinexHeader {
 			// these files are not tied to a constellation system,
 			// therefore, do not have this field
 			constellation = None
-
 		} else { // regular files
             constellation = Some(Constellation::from_str(constellation_str.trim())?)
         }
@@ -428,6 +442,7 @@ impl std::str::FromStr for RinexHeader {
             .unwrap();
         // comments ?
         while is_comment!(line) {
+            comments.push(String::from(line.split_at(60).0));
             line = lines.next()
                 .unwrap()
         }
@@ -492,6 +507,7 @@ impl std::str::FromStr for RinexHeader {
             .unwrap();
         // comments ?
         while is_comment!(line) {
+            comments.push(String::from(line.split_at(60).0));
             line = lines.next()
                 .unwrap()
         }
@@ -518,7 +534,6 @@ impl std::str::FromStr for RinexHeader {
         let mut coords     : Option<rust_3d::Point3D> = None;
         let mut epochs: (Option<gnss_time::GnssTime>, Option<gnss_time::GnssTime>) = (None, None);
         // (OBS) 
-        let obs_nb_sat : u32 = 0;
         let mut obs_codes  : HashMap<Constellation, Vec<String>> 
             = HashMap::with_capacity(constellation::CONSTELLATION_LENGTH);
 		let mut met_codes  : Vec<String> = Vec::with_capacity(3);
@@ -801,21 +816,25 @@ impl std::str::FromStr for RinexHeader {
             }
             // comments ?
             while is_comment!(line) {
+                comments.push(String::from(line.split_at(60).0));
                 line = lines.next()
                     .unwrap()
             }
         }
         
-        if ant.is_some() { // whatever.. but will avoid hypothetical crash
-            if ant_coords.is_some() { // in case of faulty producer
-                ant.as_mut().unwrap().set_coords(ant_coords.unwrap())
-            }
-            if ant_hen.is_some() {
-                ant.as_mut().unwrap().set_height(ant_hen.unwrap().0);
-                ant.as_mut().unwrap().set_eastern_eccentricity(ant_hen.unwrap().1);
-                ant.as_mut().unwrap().set_northern_eccentricity(ant_hen.unwrap().2);
-            }
-        }
+        let ant : Option<Antenna> = match ant {
+            Some(antenna) => {
+                Some(Antenna::new(
+                    &antenna.sn, 
+                    &antenna.model, 
+                    ant_coords, 
+                    Some(ant_hen.unwrap_or((0.0_f32,0.0_f32,0.0_f32)).0), 
+                    Some(ant_hen.unwrap_or((0.0_f32,0.0_f32,0.0_f32)).1), 
+                    Some(ant_hen.unwrap_or((0.0_f32,0.0_f32,0.0_f32)).2), 
+                ))
+            },
+            _ => None,
+        };
 
 		let sensors : Option<Vec<Sensor>> = match sensors.len() > 0 {
 			true => {
@@ -835,11 +854,12 @@ impl std::str::FromStr for RinexHeader {
 			false => Some(met_codes),
 		};
         
-        Ok(RinexHeader{
+        Ok(Header{
             version: version,
             crinex: crinex_infos, 
             rinex_type,
             constellation,
+            comments,
             program: String::from(pgm.trim()),
             run_by,
             station,
@@ -868,29 +888,51 @@ impl std::str::FromStr for RinexHeader {
     }
 }
 
-impl RinexHeader {
-    /// Returns true if self is a `Compressed RINEX`
-    pub fn is_crinex (&self) -> bool { self.crinex.is_some() }
-
-    /// Returns Compressed `RINEX` version (if any) 
-    pub fn get_crinex_version (&self) -> Option<&str> { 
-        match &self.crinex {
-            Some(crinex) => Some(&crinex.version),
-            _ => None,
+impl std::fmt::Display for Header {
+    /// `header` formatter, mainly for 
+    /// `RINEX` file production purposes
+    fn fmt (&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        /*if self.is_crinex() {
+            write!(f,"F9.2{}           ", self.version)?;
+            write!(f,"F9.2{}           ", self.version)?;
+        }*/
+        /*write!(f,"F9.2{}           ", self.version)?;*/
+        match self.rinex_type {
+            Type::ObservationData => {
+                write!(f,"OBSERVATION DATA    ")?
+            },
+            Type::MeteorologicalData => {
+                write!(f,"METEOROLOGICAL DATA ")?
+            },
+            Type::NavigationMessage => {
+                match self.constellation {
+                    Some(Constellation::Glonass) => write!(f,"GLONASS NAV        ")?,
+                    _ => write!(f,"NAVIGATION DATA    ")?
+                }
+            },
+            _ => {
+                panic!("non supported rinex type")
+            }
         }
-    }
-    /// Returns `RINEX` compression program name (if any) 
-    pub fn get_crinex_prog (&self) -> Option<&str> { 
-        match &self.crinex {
-            Some(crinex) => Some(&crinex.prog),
-            _ => None,
-        }
-    }
-    /// Returns `RINEX` compression date (if any) 
-    pub fn get_crinex_date (&self) -> Option<chrono::NaiveDateTime> { 
-        match &self.crinex {
-            Some(crinex) => Some(crinex.date),
-            _ => None,
+        // PGM / RUN BY / DATE
+        write!(f, "{:<20}", self.program)?;
+        write!(f, "{:<20}", self.run_by)?; // TODO date field parsing
+        write!(f, "{:<20}", "PGM / RUN BY / DATE\n")?; 
+        // OBSERVER / AGENCY
+        write!(f, "{:<20}", self.observer)?;
+        write!(f, "{:<40}", self.agency)?;
+        write!(f, "{:<18}", "OBSERVER / AGENCY\n")?; 
+        // COMMENTS 
+        for comment in self.comments.iter() {
+            write!(f, "{:<60}", comment)?;
+            write!(f, "COMMENT\n")?
         }
+        // END OF HEADER
+        write!(f, "{:>73}", "END OF HEADER")
     }
 }
+
+impl Header {
+    /// Returns true if self is a `Compressed RINEX`
+    pub fn is_crinex (&self) -> bool { self.crinex.is_some() }
+}
