@@ -4,19 +4,21 @@
 //! Refer to README for example of use.  
 //! Homepage: <https://github.com/gwbres/rinex>
 mod meteo;
-mod header;
-mod version;
 mod gnss_time;
 mod navigation;
 mod observation;
 
+pub mod types;
 pub mod epoch;
+pub mod header;
 pub mod record;
+pub mod version;
 pub mod hatanaka;
 pub mod constellation;
 
 use thiserror::Error;
 use std::str::FromStr;
+use std::io::Write;
 
 #[macro_export]
 /// Returns `true` if given `Rinex` line is a comment
@@ -24,66 +26,11 @@ macro_rules! is_comment {
     ($line: expr) => { $line.contains("COMMENT") };
 }
 
-/// Describes all known `RINEX` file types
-#[derive(Copy, Clone, PartialEq, Debug)]
-pub enum Type {
-    /// Describes Observation Data (OBS),
-    /// Phase & Pseudo range measurements
-    ObservationData, 
-    /// Describes Navigation Message (NAV)
-    /// Ephemeride file
-    NavigationMessage,
-    /// Describes Meteorological data (Meteo)
-    MeteorologicalData,
-}
-
-#[derive(Error, Debug)]
-/// `Type` related errors
-pub enum TypeError {
-    #[error("Unknown RINEX type identifier \"{0}\"")]
-    UnknownType(String),
-}
-
-impl Default for Type {
-    /// Builds a default `Type`
-    fn default() -> Type { Type::ObservationData }
-}
-
-impl Type {
-    /// Converts `Self` to str
-    pub fn to_str (&self) -> &str {
-        match *self {
-            Type::ObservationData => "ObservationData",
-            Type::NavigationMessage => "NavigationMessage",
-            Type::MeteorologicalData => "MeteorologicalData",
-        }
-    }
-    /// Converts `Self` to string
-    pub fn to_string (&self) -> String { String::from(self.to_str()) }
-}
-
-impl std::str::FromStr for Type {
-    type Err = TypeError;
-    fn from_str (s: &str) -> Result<Self, Self::Err> {
-        if s.eq("NAVIGATION DATA") {
-            Ok(Type::NavigationMessage)
-        } else if s.contains("NAV DATA") {
-            Ok(Type::NavigationMessage)
-        } else if s.eq("OBSERVATION DATA") {
-            Ok(Type::ObservationData)
-        } else if s.eq("METEOROLOGICAL DATA") {
-            Ok(Type::MeteorologicalData)
-        } else {
-            Err(TypeError::UnknownType(String::from(s)))
-        }
-    }
-}
-
 /// `Rinex` describes a `RINEX` file
 #[derive(Debug)]
 pub struct Rinex {
     /// `header` field contains general information
-    pub header: header::RinexHeader,
+    pub header: header::Header,
     /// `record` contains `RINEX` file body
     /// and is type and constellation dependent 
     pub record: Option<record::Record>,
@@ -93,7 +40,7 @@ impl Default for Rinex {
     /// Builds a default `RINEX`
     fn default() -> Rinex {
         Rinex {
-            header: header::RinexHeader::default(),
+            header: header::Header::default(),
             record: None, 
         }
     }
@@ -107,12 +54,12 @@ pub enum RinexError {
     #[error("Header parsing error")]
     HeaderError(#[from] header::Error),
     #[error("Rinex type error")]
-    TypeError(#[from] TypeError),
+    TypeError(#[from] types::TypeError),
 }
 
 impl Rinex {
     /// Builds a new `RINEX` struct from given:
-    pub fn new (header: header::RinexHeader, record: Option<record::Record>) -> Rinex {
+    pub fn new (header: header::Header, record: Option<record::Record>) -> Rinex {
         Rinex {
             header,
             record,
@@ -134,25 +81,31 @@ impl Rinex {
     }
 
     /// Retruns true if this is an NAV rinex
-    pub fn is_navigation_rinex (&self) -> bool { self.header.rinex_type == Type::NavigationMessage }
+    pub fn is_navigation_rinex (&self) -> bool { self.header.rinex_type == types::Type::NavigationMessage }
     /// Retruns true if this is an OBS rinex
-    pub fn is_observation_rinex (&self) -> bool { self.header.rinex_type == Type::ObservationData }
+    pub fn is_observation_rinex (&self) -> bool { self.header.rinex_type == types::Type::ObservationData }
     /// Returns true if this is a METEO rinex
-    pub fn is_meteo_rinex (&self) -> bool { self.header.rinex_type == Type::MeteorologicalData }
+    pub fn is_meteo_rinex (&self) -> bool { self.header.rinex_type == types::Type::MeteorologicalData }
 
-    /// Builds a `Rinex` from given file.
-    /// Input file must respect the whitespace specifications
-    /// for the entire header section.   
-    /// The header section must respect the labelization standard too.
+    /// Builds a `RINEX` from given file.
+    /// Header section must respect labelization standards,   
+    /// some are mandatory.   
+    /// Parses record for supported `RINEX` types
     pub fn from_file (fp: &std::path::Path) -> Result<Rinex, RinexError> {
         let (header, body) = Rinex::split_rinex_content(fp)?;
-        let header = header::RinexHeader::from_str(&header)?;
+        let header = header::Header::from_str(&header)?;
         let record : Option<record::Record> = match header.is_crinex() {
             false => Some(record::build_record(&header,&body)?),
             true => None,
         };
         Ok(Rinex { header, record })
     }
+
+    /// Writes self into given file
+    pub fn to_file (&self, path: &str) -> std::io::Result<()> {
+        let mut writer = std::fs::File::create(path)?;
+        write!(writer, "{}", self.header.to_string())
+    }
 }
 
 #[cfg(test)]
@@ -217,4 +170,13 @@ mod test {
             }
         }
     }
+    #[test]
+    /// Tests `Rinex` production method 
+    fn test_rinex_production() {
+        let mut header = header::Header::default();
+        header.comments.push(String::from("THIS FILE WAS PRODUCED BY SELFTESTS"));
+        header.comments.push(String::from("     for testing purposes"));
+        let rinex = Rinex::new(header, None);
+        rinex.to_file("test").unwrap()
+    }
 }
