@@ -4,11 +4,10 @@ use std::str::FromStr;
 use std::collections::HashMap;
 
 use crate::epoch;
-use crate::version;
 use crate::record;
 use crate::record::Sv;
 use crate::constellation;
-use crate::header::RinexHeader;
+use crate::header::Header;
 
 include!(concat!(env!("OUT_DIR"),"/nav_data.rs"));
 
@@ -94,7 +93,7 @@ pub enum RecordError {
 }
 
 /// Builds `RinexRecord` entry for `NavigationMessage` file
-pub fn build_record_entry (header: &RinexHeader, content: &str)
+pub fn build_record_entry (header: &Header, content: &str)
         -> Result<(epoch::Epoch, Sv, HashMap<String, ComplexEnum>), RecordError>
 {
     //  <o 
