@@ -9,7 +9,7 @@ use crate::epoch;
 use crate::record;
 use crate::record::Sv;
 use crate::constellation;
-use crate::header::RinexHeader;
+use crate::header::Header;
 
 #[macro_export]
 /// Returns True if 3 letter code 
@@ -77,9 +77,7 @@ pub type Record = HashMap<epoch::Epoch,
 /// rcvr_clock_offset: receiver clock offset (s)    
 /// sat_clock_offset: Sv clock offset (s)    
 /// biases: other additive biases
-pub fn distance_from_pseudo_range (pr: f64,
-    rcvr_clock_offset: f64, sat_clock_offset: f64, biases: Vec<f64>)
-        -> f64 {
+pub fn distance_from_pseudo_range (pr: f64, rcvr_clock_offset: f64, sat_clock_offset: f64, biases: Vec<f64>) -> f64 {
     pr - SPEED_OF_LIGHT_IN_VACUUM * (rcvr_clock_offset - sat_clock_offset)
     // modulo leap second?
     // p17 table 4
@@ -105,7 +103,7 @@ pub enum RecordError {
 /// Builds `RINEX` record entry for `Observation` Data files.    
 /// Returns identified `epoch` to later sort data efficiently.    
 /// Returns 2D data as described in `record` definition
-pub fn build_record_entry (header: &RinexHeader, content: &str)
+pub fn build_record_entry (header: &Header, content: &str)
         -> Result<(epoch::Epoch, Option<f32>, HashMap<Sv, HashMap<String, ObservationData>>), RecordError> 
 {
     let mut lines = content.lines();
@@ -327,7 +325,7 @@ pub fn build_record_entry (header: &RinexHeader, content: &str)
 		} // for all systems
     } // V < 3 old fashion
 	else { // V > 2 modern RINEX
-		for i in 0..n_sat {
+		for _ in 0..n_sat {
 			if let Some(l) = lines.next() {
 				line = l;
 			} else {
