@@ -3,12 +3,12 @@ use thiserror::Error;
 use std::collections::HashMap;
 
 use crate::epoch;
-use crate::header;
 use crate::meteo;
+use crate::header;
 use crate::navigation;
 use crate::observation;
 use crate::is_comment;
-use crate::{Type, TypeError};
+use crate::types::{Type, TypeError};
 use crate::constellation::Constellation;
 
 /// ̀`Sv` describes a Satellite Vehiculee
@@ -114,12 +114,18 @@ pub enum RecordError {
     TypeError(String),
 }
 
-/// Returns true if a new block record section
-/// is encountered at given line.
-/// Block record section represents an epoch for most RINEX files
-fn block_record_start (line: &str, header: &header::RinexHeader) -> bool {
+/// Returns true if given line matches the start   
+/// of a new epoch, inside a RINEX record.    
+/// Will panic on CRINEX data - unable to hanle it
+pub fn is_new_epoch (line: &str, header: &header::Header) -> bool {
     let parsed: Vec<&str> = line.split_ascii_whitespace()
         .collect();
+    if header.is_crinex() {
+        panic!("is_new_epoch() for CRINEX record is not supported")
+    }
+    if is_comment!(line) {
+        return false
+    }
 	match header.version.major {
 		1|2|3 => {
 			// old RINEX
@@ -183,8 +189,6 @@ fn block_record_start (line: &str, header: &header::RinexHeader) -> bool {
 						},
 					}
 				},
-				_ => false, // non supported, 
-					// unknown RINEX types
 			}
 		},
 		_ => {
@@ -213,7 +217,7 @@ fn block_record_start (line: &str, header: &header::RinexHeader) -> bool {
 
 /// Builds a `Record`, `RINEX` file body content,
 /// which is constellation and `RINEX` file type dependent
-pub fn build_record (header: &header::RinexHeader, body: &str) -> Result<Record, TypeError> { 
+pub fn build_record (header: &header::Header, body: &str) -> Result<Record, TypeError> { 
     let mut body = body.lines();
     let mut line = body.next()
         .unwrap();
@@ -230,7 +234,7 @@ pub fn build_record (header: &header::RinexHeader, body: &str) -> Result<Record,
 	let mut met_rec : meteo::Record = HashMap::new();
     
     loop {
-        let is_new_block = block_record_start(&line, &header);
+        let is_new_block = is_new_epoch(&line, &header);
         if is_new_block && !first {
             match &header.rinex_type {
                 Type::NavigationMessage => {
@@ -250,7 +254,6 @@ pub fn build_record (header: &header::RinexHeader, body: &str) -> Result<Record,
 						met_rec.insert(e, map);
 					}
 				},
-                _ => unreachable!(), // unknown file type
             }
         }
 
@@ -286,7 +289,6 @@ pub fn build_record (header: &header::RinexHeader, body: &str) -> Result<Record,
         Type::NavigationMessage => Ok(Record::NavRecord(nav_rec)),
         Type::ObservationData => Ok(Record::ObsRecord(obs_rec)), 
 		Type::MeteorologicalData => Ok(Record::MeteoRecord(met_rec)),
-		_ => unreachable!(),
         //_ => Err(TypeError::UnknownType(header.rinex_type.to_string())),
     }
 }
